﻿using static System.Environment;
using System.ClientModel;
using Azure.AI.OpenAI;
using OpenAI.Chat;
using DotNetEnv;
using System.ClientModel.Primitives;

Env.Load();

string endpoint = GetEnvironmentVariable("AZURE_OPENAI_ENDPOINT");
string key = GetEnvironmentVariable("AZURE_OPENAI_API_KEY");
string subscriptionKey = GetEnvironmentVariable("AZURE_APIM_API_KEY");

ApiKeyCredential apiKeyCredential = new ApiKeyCredential(key);
AzureOpenAIClientOptions clientOptions = new AzureOpenAIClientOptions();

PipelinePolicy customHeaders = new GenericActionPipelinePolicy(requestAction: request =>
{
    request.Headers.Set("Ocp-Apim-Subscription-Key", subscriptionKey);
});

clientOptions.AddPolicy(customHeaders, PipelinePosition.PerCall);

AzureOpenAIClient azureClient = new(
    new Uri(endpoint),
    apiKeyCredential,
    clientOptions);

ChatClient chatClient = azureClient.GetChatClient("gpt-4o");
try
{
    ChatCompletion completion = chatClient.CompleteChat(
        new ChatMessage[]
        {
            new SystemChatMessage("You are a helpful assistant that talks like a pirate."),
            new UserChatMessage("Good day, who am I talking to?"),
            new AssistantChatMessage("Ahoy there, matey! What be bringin ye to these waters today? "),
            new UserChatMessage("Where is the treasure island?")
        });

    Console.WriteLine(completion);
    Console.WriteLine($"{completion.Role}: {completion.Content[0].Text}");
}
catch (Exception ex)
{
    Console.WriteLine($"An error occurred: {ex.Message}");
}


internal class GenericActionPipelinePolicy : PipelinePolicy
{
    private Action<PipelineRequest> _requestAction;
    private Action<PipelineResponse> _responseAction;

    public GenericActionPipelinePolicy(Action<PipelineRequest> requestAction = null, Action<PipelineResponse> responseAction = null)
    {
        _requestAction = requestAction;
        _responseAction = responseAction;
    }

    public override void Process(PipelineMessage message, IReadOnlyList<PipelinePolicy> pipeline, int currentIndex)
    {
        _requestAction?.Invoke(message.Request);
        ProcessNext(message, pipeline, currentIndex);
        _responseAction?.Invoke(message.Response);
    }

    public override async ValueTask ProcessAsync(PipelineMessage message, IReadOnlyList<PipelinePolicy> pipeline, int currentIndex)
    {
        _requestAction?.Invoke(message.Request);
        await ProcessNextAsync(message, pipeline, currentIndex).ConfigureAwait(false);
        _responseAction?.Invoke(message.Response);
    }
}
